"""
Adapted from https://github.com/aypro-droid/image-to-ascii

MIT License

Copyright (c) 2021 Not Aypro

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

try:
    import Image as Image
except ImportError:
    import PIL
    from PIL import Image as Image

from collections import Counter


ASCII_CHARS = ["@", "#", "S", "%", "?", "*", "+", ";", ":", ",", "."]


class ImageToAscii:
    def __init__(self, imagePath: str = None, width: int = 100, outputFile: str = None):
        """
        path - The path/name of the image ex: <image_name.png>\n
        width - The width you want the Ascii art to have\n
        outputfile - If you want to store the Ascii art in a txt file then set it to <file_name.txt> else keep it None
        """
        self.path = imagePath
        self.width = width
        try:
            self.image = PIL.Image.open(self.path)
            width, height = self.image.size
            totalPixels = width * height
        except:
            if imagePath is None:
                print("Invalid path name")
            elif self.width is None:
                print("Invalid Width provided")
        self.new_image_data = self.pixelsToAscii(
            self.converToGrayscale(self.resizeImage(self.image))
        )

        self.pixel_count = len(self.new_image_data)

        self.ascii_image = "\n".join(
            [
                self.new_image_data[index : (index + self.width)]
                for index in range(0, self.pixel_count, self.width)
            ]
        )
        if outputFile is not None:
            with open(outputFile, "w") as f:
                f.write(self.ascii_image)

    def resizeImage(self, image):
        width, height = image.size
        ratio = height / width
        new_height = int(self.width * ratio)
        resized_image = image.resize((self.width, new_height))

        return resized_image

    def converToGrayscale(self, image):
        grayscale_image = image.convert("L")
        return grayscale_image

    def pixelsToAscii(self, image):
        pixels = image.getdata()
        characters = "".join([ASCII_CHARS[pixel // 25] for pixel in pixels])
        return characters

    def clean(self):
        def count(s, total):
            c = Counter(s)
            _blank = c.get(".", 0) + c.get(",", 0)
            _others = total - _blank
            return _blank, _others

        result = []
        for line in self.ascii_image.split("\n"):
            blank, others = count(line.strip(), self.width)
            if others >= self.width / 40.0:
                result.append(line)
        height = len(result)
        start = 0
        end = self.width
        masks = [True] * self.width
        for i in range(0, self.width):
            blank, others = count([result[j][i] for j in range(height)], height)
            if others > 0:
                end = i + 1
                for j in range(start + 2, end - 2):
                    masks[j] = False
                start = i + 1
        end = self.width
        for j in range(start + 2, end - 2):
            masks[j] = False

        new_result = []
        for line in result:
            new_line = ""
            for i in range(self.width):
                if masks[i]:
                    if line[i] == "." or line[i] == ",":
                        new_line += " "
                    else:
                        new_line += line[i]
            new_result.append(new_line)

        return "\n".join(new_result)



from datetime import datetime
from io import BytesIO
from types import TracebackType
from typing import Optional, Type, Dict, Any
from http.cookies import SimpleCookie
from urllib.parse import urlparse

from aiohttp import ClientSession
from bs4 import BeautifulSoup
from jaccount_auth.ascii import ImageToAscii, Image

# make the jaccount server believe it is a browser
JACCOUNT_HEADERS = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/91.0.4472.77 Safari/537.36",
    "Accept-Language": "zh-CN,zh;q=0.9",
    "Referer": "https://jaccount.sjtu.edu.cn/",
}
CAPTCHA_URL = "https://jaccount.sjtu.edu.cn/jaccount/captcha"
LOGIN_URL = "https://jaccount.sjtu.edu.cn/jaccount/ulogin"


class JaccountCLIAsyncIO:
    def __init__(self, base_url: str, session: Optional[ClientSession] = None):
        if session:
            self.session = session
            self._session_autoclose = False
        else:
            self.session: ClientSession = ClientSession(headers=JACCOUNT_HEADERS)
            self._session_autoclose = True
        self.base_url: str = base_url
        self.cookies: Optional[SimpleCookie[str]] = None
        self.captcha_image: Optional[BytesIO] = None
        self.params: Dict[str, Any] = {}

    async def __aenter__(self) -> "JaccountCLIAsyncIO":
        return self

    async def __aexit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None:
        if self._session_autoclose:
            await self.session.close()

    async def init(self):
        async with self.session.get(self.base_url) as response:
            body = await response.text()
            self.cookies = response.cookies
        soup = BeautifulSoup(body, "html.parser")
        hidden_inputs = soup.select("input[type=hidden]")
        self.params = dict({(x["name"], x["value"]) for x in hidden_inputs})
        url = "%s?uuid=%s&t=%d" % (
            CAPTCHA_URL,
            self.params["uuid"],
            int(datetime.now().timestamp()),
        )
        async with self.session.get(url, cookies=self.cookies) as response:
            fp = BytesIO()
            fp.write(await response.read())
            self.captcha_image = fp

    async def login(self, username, password, captcha):
        self.params["v"] = ""
        self.params["user"] = username
        self.params["pass"] = password
        self.params["captcha"] = captcha

        async with self.session.post(
            LOGIN_URL, cookies=self.cookies, data=self.params
        ) as response:
            pass

    def captcha_show_external(self):
        image = Image.open(self.captcha_image)
        image.show()

    def captcha_generate_ascii(self):
        image_to_ascii = ImageToAscii(self.captcha_image)
        return image_to_ascii.clean()

    def get_cookies(self):
        url = urlparse(self.base_url)
        domain_url = "{}://{}".format(url.scheme, url.netloc)
        cookies = self.session.cookie_jar.filter_cookies(domain_url)
        return cookies



from setuptools import setup


def get_long_description():
    """
    Return the README.
    """
    return open("README.md", "r", encoding="utf8").read()


setup(
    name="jaccount-auth",
    version="0.0.1",
    url="https://github.com/linsyking/jaccount-auth",
    license="MIT",
    description="A small plugin to help use jaccount login in python programs.",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author="linsyking",
    author_email="xiangyiming2002@gmail.com",
    maintainer="linsyking",
    maintainer_email="xiangyiming2002@gmail.com",
    packages=["jaccount_auth"],
    include_package_data=True,
    project_urls={
        "Bug Reports": "https://github.com/linsyking/jaccount-auth/issues",
        "Source": "https://github.com/linsyking/jaccount-auth",
    },
    install_requires=[
        "aiohttp",
        "beautifulsoup4",
        "pillow",
    ],
)
